from vcCommand import *
import re
import os.path

app = getApplication()

def first_state():
  # FIRST STATE CREATES THE UI FOR THE USER
  global group, uri_xml_base, uri_csv,cvsCompList, vcidDict, lookForProps, base
  global add_line_c_prop, remove_line_c_prop, generate_prop, write_xml_prop, read_xml_prop, server_variable_prop
  # REMOVE EXISTING COMMAND PROPERTIES
  cmd = getCommand()
  cmdProps = cmd.Properties
  movers, mover_type = execute_action(None)
  if movers is None or mover_type is None or not len(movers):
    print 'Could not determine movers from selection. Please select XPlanar or XTS movers.'
    return
  movers.sort(key= lambda x: natural_keys(x.Name))
  # READ URI
  uri = getCommandPath()
  base, filename = os.path.split(uri)

  uri_xml_base = os.path.join(base, 'template/ConfigurationBase.xml')
  uri_xml_base = uri_xml_base[8:]

  # CREATE PROPERTIES
  group = 0
  group2 = 0

  server_variable_prop = createProperty(VC_STRING, 'Server Variable Name')
  server_variable_prop.Group = group2 = group2 + 500
  print "Please enter the server variable format for movers in the PLC project. Array starting index is indicated in curly brackets."
  if mover_type == 'XTS':
    server_variable_prop.Value = "MAIN.Mover[{0}].AxisReference.NcToPlc.ActPos" #Port 851, default mover start index = 0
    #server_variable_prop.Value = "Axes.Mover Axis {1}.ActPos" #Port 501, default mover start index = 1
    #server_variable_prop.Value = "Mover Axis {1}.SoftDrive {1}.SdScopeVariable.ActPos" #Port 351, default mover start index = 1
  elif mover_type == 'XPlanar':
    server_variable_prop.Value = "MAIN.fbMover[{0}].MCTOPLC.ACT.ActPos"#Port 851, default mover start index = 0

  read_xml_prop = createProperty(VC_BUTTON, 'Generate Configuration XML')
  read_xml_prop.Group = group2 = group2 + 500
  read_xml_prop.OnChanged = event_wrapper(movers, mover_type, server_variable_prop)

  executeInActionPanel() 


def execute_action(arg):
  sm = app.SelectionManager
  selection = sm.getSelection(VC_SELECTION_COMPONENT)
  #Viable selections: XTS Mover, XTS Controller, XPlanar mover
  movers = []
  mover_type = ''
  for c in selection:
    mover_ife = c.findBehaviour('MoverInterface')
    controller_ife = c.findBehaviour('ControllerInterface')
    mover_node = c.findNode('Mover')
    if mover_ife is not None:
      #XTS controller
      #print 'Controller selected', selection
      if mover_type == 'XPlanar':
        print 'Selection cannot contain both XTS and XPlanar components'
        return
      movers = mover_ife.ConnectedComponents
      mover_type = 'XTS'
      break
    elif controller_ife is not None:
      #XTS mover
      #print 'XTS Mover selected', selection
      if mover_type == 'XPlanar':
        print 'Selection cannot contain both XTS and XPlanar components'
        return
      mover_type = 'XTS'
      movers.append(c)
    elif mover_node is not None:
      #XPlanar mover
      #print 'XPlanar mover selected', selection
      if mover_type == 'XTS':
        print 'Selection cannot contain both XTS and XPlanar components'
        return
      mover_type = 'XPlanar'
      movers.append(c)
  return movers, mover_type


def event_wrapper(movers, mover_type, server_variable_prop):
  def inner(arg):
    writeXML(movers, mover_type, server_variable_prop)
  return inner


#Creates a XML configuration file
def writeXML(movers, mover_type, server_var_prop):
  '''# reads XML and generates the line '''
  text = ""
  with open (uri_xml_base, "r") as template_file:
    text = template_file.read()
  if len(text) == 0:
    print 'Could not read file'
    return
  server_var = server_var_prop.Value
  start_index, curlies = get_start_index(server_var)
  if start_index is None or start_index == -1:
    return
  if mover_type == "XTS":
    item_text = pair_XTS(movers, server_var, start_index, curlies)
  elif mover_type == "XPlanar":
    item_text = pair_XPlanar(movers, server_var, start_index, curlies)
  else:
    print 'Invalid selection'
    return
  text = text.replace("{ITEMHERE}", item_text)
  uri_xml = os.path.join(base, "ServerConfiguration.xml")
  uri_xml = uri_xml[8:]
  #if os.path.isfile(uri_xml):
  with open (uri_xml, "wb") as outfile:
    outfile.write(text.encode('utf8'))

  print "Output file written to:", uri_xml
  print "Select Import from the Configuration toolbar to use this setup."


#Generate pairing XML text for XTS movers
def pair_XTS(movers, server_var, start_index, curlies):
  prop = "ActPos"
  item_text = ""
  pairs = []
  for i, mover in enumerate(movers, start_index):
    item_text += XTS_base.format(ServerVariableName = server_var.replace(curlies, str(i)), ComponentName = mover.Name, PropertyName = prop)
    pairs.append([server_var.replace(curlies, str(i)), mover.Name, prop])
    if i < len(movers) - start_index - 1:
      item_text += "\n\t\t\t\t\t\t\t"
  if len(pairs):
    print "Pairings created with the following parameters:"
    for pair in pairs:
      print "{} -- {}: {}".format(pair[0], pair[1], pair[2])
    else:
      "Failed to pair variables."
  return item_text


#Generate pairing XML text for XPlanar movers
def pair_XPlanar(movers, server_var, start_index, curlies):
  prop = "ActPos"
  item_text = ""
  pairs = []
  dofs = ["x", "y", "z", "a", "b", "c"]
  for i, mover in enumerate(movers, start_index):
    for dof in dofs:
      item_text += XPlanar_base.format(ServerVariableName = server_var.replace(curlies, str(i)), ComponentName = mover.Name, DOF = dof.upper(), dof = dof)
      if i < len(movers) - start_index - 1 and dof != "c":
        item_text += "\n\t\t\t\t\t\t\t"
    pairs.append([server_var.replace(curlies, str(i)), mover.Name])
  if len(pairs):
    print "Pairings created with the following parameters:"
    for pair in pairs:
      print "{} -- {}".format(pair[0], pair[1])
    else:
      "Failed to pair variables."
  return item_text


XTS_base = """ <Array:anyType i:type="VariableGroupItem">
                  <DisplayName i:nil="true" />
                  <ServerItem i:type="System:string">{ServerVariableName}</ServerItem>
                  <SimulationVariable>
                    <Component>{ComponentName}</Component>
                    <Behaviour i:nil="true" />
                    <Property>{PropertyName}</Property>
                  </SimulationVariable>
                </Array:anyType>"""

XPlanar_base = """<Array:anyType i:type="VariableGroupItem">
                  <DisplayName i:nil="true" />
                  <ServerItem i:type="System:string">{ServerVariableName}.{dof}</ServerItem>
                  <SimulationVariable i:type="SimulationDofVariable">
                    <Component>{ComponentName}</Component>
                    <Behaviour>Controller</Behaviour>
                    <Property>VALUE</Property>
                    <DofObject>{DOF}</DofObject>
                  </SimulationVariable>
                </Array:anyType>"""


#Extract the starting index for mover server variables
def get_start_index(server_var):
  pattern = re.compile(r"{(\d+)}")
  start_index_list = re.findall(pattern, server_var)
  if len(start_index_list) > 0:
    if len(set(start_index_list)) != 1: #Check if all found elements are same
      print r'Failed to extract server indices. Please mark starting inex with square brackets. Example: Axis {1}'
      return -1, None
    start_index = int(start_index_list[0])
    curlies = "{" + str(start_index) + "}"
  else:
    pattern = re.compile(r"{}")
    start_index_list = re.findall(pattern, server_var)
    if len(start_index_list) == 0:
      print r'Failed to extract server indices. Please mark starting inex with square brackets. Example: Axis {1}'
      return -1, None
    start_index = 0
    curlies = r"{}"
  return start_index, curlies


#Helper for sorting movers by name
def atoi(text):
    return int(text) if text.isdigit() else text

#Sorting movers by name
def natural_keys(text):
    return [ atoi(c) for c in re.split(r'(\d+)', text) ]


addState(first_state)